/***************************************************************************\

   Copyright (C) 1989-1994   Lotus Development Corporation.
   All rights reserved.

   File Name:  commport.c

   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
       the source code version and to reproduce and distribute the object
       code version of the Example Programs provided in the Toolkit, provided
       that you:

       a) distribute the Example Programs only in conjunction with and as a
          part of your software application product which is designed as an
          add-in to Lotus 1-2-3, and

       b) do not represent your product to be a product of Lotus Development
          Corporation.

   2.  The Example Programs are provided as is, without warranty of any kind,
       either express or implied, including, without limitation, the implied
       warranties of merchantability of fitness for a particular purpose.

\***************************************************************************/


#include "commport.plh"
#include "lpixport.h"
#include "string.h"
#if defined(DEBUGON)
#include "debug.h"
#endif

/*-----------------------------------------------------------------
-         Generally useful Macros
-----------------------------------------------------------------*/
#define ALLOC_REAL      (*LpiContext->Environment->alloc)
#define FREE_REAL       (*LpiContext->Environment->dealloc)
#define LIB_DATA        ((lptr(library_data))LpiContext->LibData)

/* use this to get an input instance data for SET attribute */
/* GET INSTANCE, MAKE COPY IF IT IS *NOT* ALIASED */
#define GET_IN_ADT(xadt)   if ((!LpiAliased(PLinput(0))) &&\
         (((lptr(CommportType))LpiMapMem(xadt, VMR_1))->ref_count > 1)) {\
         LpiUnmapMem(xadt, VMR_1);\
         lpi_cp_copy(LpiContext, xadt, &xadt);}

/* use this to return instance data for SET attribute */
/* IF WE WERE SUCCESSFUL AND THINGS WORKED PROPERLY */
#define SET_OUT_ADT(xadt)  LpiCopyData(PLoutput(0),PLinput(0));\
         if (xadt != LpiGetInADTdata(0)){LpiSetOutADTdata( 0, xadt );}\
         else {if (!LpiAliased(PLinput(0))) {\
         ((lptr(CommportType))LpiMapMem(p_thePort, VMR_1))->ref_count++;\
         LpiUnmapMem(p_thePort, VMR_1);}}

/* use this to destroy instance data if error in SET procedure */
/* IF THERE WAS AN ERROR AND WE ARE NOT RETURNING THE INSTANCE */
#define DESTROY_INSTANCE(xadt)   if (xadt != LpiGetInADTdata(0)){\
                                 lpi_cp_destroy( LpiContext, &xadt );}

/*-----------------------------------------------------------------
-         General #defines
-----------------------------------------------------------------*/
#define PARITY_ODD      0
#define PARITY_EVEN     1
#define PARITY_NONE     2
#define BUFFER_LENGTH   2048        /* buffer for port to write */

/*-----------------------------------------------------------------
-         Instance data and other typedefs
-----------------------------------------------------------------*/
/* commport ADT instance data structure */
typedef struct CommportType_{
   lsshort     ref_count;        /* reference count holder */
   lsshort     baud_rate;
   lsshort     port_number;      /* ie. com1, com2, com3... */
   lsshort     data_bits;        /* 7 or 8 allowed */
   lsshort     stop_bits;        /* 1 or 2 allowed */
   lsshort     parity;           /* ODD, EVEN, NONE */
   lbool       port_open;        /* flag that the port is open */
   lptr(lmbcs) port_buffer;      /* input buffer for port LNULL if closed */
   lmhdl(CommportType_)   next;  /* point to next element in list */
}CommportType;

typedef lmhdl(CommportType)  LpiCommportType;

/* Commport ADT utility vector / linked list structure */
typedef struct {
   LpiCommportType   port_list;        /* head of singly linked list */
   lmhdl(LpiADTUTIL) commport_vector;  /* utility vector */
} library_data ;

/* AltExportType is the name of the typedef'ed table created for your
 * alternate (C callable) exported procedures in the .PLX file.  The name
 * must be identical for the alternate exported procedure table to compile
 * correctly in the bind routine.
 */
#define AltExportType cp_exports

/*************************************************************************\
   INCLUDES: included here because it relies on definitions above....
\*************************************************************************/
#include "commport.plx"

/*-----------------------------------------------------------------
-         Prototypes for LPL callable functions
-----------------------------------------------------------------*/
/* BIND/UNBIND */
LpiBindProto   (commport_b);
LpiUnbindProto (commport_u);

/* UTILITY VECTOR FUNCTIONS */
LpiProto (destroy_commport);
LpiProto (duplicate_commport);
LpiProto (display_commport);
LpiProto (compare_commport);

/* CONSTRUCTOR, ATTRIBUTES, PROCEDURES, OPERATORS */
LpiProto ( construct_commport );
LpiProto ( baudrate_set );
LpiProto ( baudrate_get );
LpiProto ( port_set );
LpiProto ( port_get );
LpiProto ( databits_set );
LpiProto ( databits_get );
LpiProto ( stopbits_set );
LpiProto ( stopbits_get );
LpiProto ( parity_set );
LpiProto ( parity_get );
LpiProto ( open );
LpiProto ( close );
LpiProto ( select );
LpiProto ( replace );

/************************************************************************\
   Local Function Prototypes
\************************************************************************/
lsshort static near lpi_cp_copy ( lptr(CONTEXT) LpiContext,
                                  LpiCommportType oldCommport,
                                  lptr(LpiCommportType) p_newCommport);

lbool static near not_open ( lptr(CONTEXT) LpiContext,
                                  lsshort  port_number);

void static near remove_from_list ( lptr(CONTEXT) LpiContext,
                                    LpiCommportType   p_theCommport);

void static near add_to_list ( lptr(CONTEXT) LpiContext,
                                    LpiCommportType   p_theCommport);


/************************************************************************\
   BIND FUNCTION
\************************************************************************/

LpiBind(commport_b)
{   /* binding proc */

   lptr(LpiADTUTIL)  util_vec;

/*-----------------------------------------------------------------
-        Exported (LPL callable) Procedures Table
-
-  These functions must appear in the same sequence they appear in
-  the LPL procedure declaration file.
-
-----------------------------------------------------------------*/
   LpiSetBegin
      LpiSet ( construct_commport )
      LpiSet ( display_commport )
               LpiSet ( baudrate_set )
               LpiSet ( baudrate_get )
               LpiSet ( port_set )
               LpiSet ( port_get )
               LpiSet ( databits_set )
               LpiSet ( databits_get )
               LpiSet ( stopbits_set )
               LpiSet ( stopbits_get )
               LpiSet ( parity_set )
               LpiSet ( parity_get )
               LpiSet ( open )
               LpiSet ( close )
      LpiSet ( select )
      LpiSet ( replace )
      LpiSet ( compare_commport )
   LpiSetEnd

/*-----------------------------------------------------------------
-        Alternate (C callable) Exported Procedures Table
-
-  These functions must appear in the same order they appear in the
-  .PLX file export function table.
-
-----------------------------------------------------------------*/
   LpiAltSetBegin
      LpiAltSet   (lpi_cp_construct)
      LpiAltSet   (lpi_cp_destroy)
      LpiAltSet   (lpi_cp_string)
      LpiAltSet   (lpi_cp_set_baudrate)
      LpiAltSet   (lpi_cp_get_baudrate)
      LpiAltSet   (lpi_cp_set_port)
      LpiAltSet   (lpi_cp_get_port)
      LpiAltSet   (lpi_cp_set_databits)
      LpiAltSet   (lpi_cp_get_databits)
      LpiAltSet   (lpi_cp_set_stopbits)
      LpiAltSet   (lpi_cp_get_stopbits)
      LpiAltSet   (lpi_cp_set_parity)
      LpiAltSet   (lpi_cp_get_parity)
      LpiAltSet   (lpi_cp_open)
      LpiAltSet   (lpi_cp_close)
      LpiAltSet   (lpi_cp_select)
      LpiAltSet   (lpi_cp_replace)
   LpiAltSetEnd


   /* Verify that we match the time and version stamps, then export
    * the two tables created above.
    */
   LpiSetCode( COMMPORT_STAMP_0, COMMPORT_STAMP_1, COMMPORT_VERSION );
   LpiAltSetCode;


   /*------------------------------------------------------------------*\
    - Setup Utility vector
   \*------------------------------------------------------------------*/
#if defined(DEBUGON)
   debug();
#endif

   /* allocate space for library data LibData element of LpiContext */
   LpiContext -> LibData = ALLOC_REAL ( (lslong)sizeof(library_data) );

   /* check for allocation failure (null pointer) and return if found */
   if ( LpiContext->LibData == LNULL )
         LpiSignal( LPI_FAILURE );

   /* initialize linked list and Utility vector */
   LIB_DATA->port_list = LNULL;     /* no instances, NULL head pointer of
                                     * linked list.
                                     */

   /* Allocate handle to utility vector.  The utility vector must ALWAYS
    * be a handle, not a real pointer.
    */
   LIB_DATA->commport_vector = LpiAllocMem ( LpiADTUTIL,
                                             sizeof(LpiADTUTIL),
                                             VMR_0);

   /* Test to assure that utility vector allocation succeeded */
   if (LpiCheckMem(LIB_DATA->commport_vector)) {
      /* if it failed, free the LibData pointer and return failure */
      FREE_REAL( LpiContext->LibData, (lslong)sizeof(library_data));
      LpiSignal (LPI_FAILURE);
   }

   /* Map the utility vector so we can initialize it */
   util_vec = (lptr(LpiADTUTIL))LpiMapMem (LIB_DATA->commport_vector, VMR_0);

   /* initialize the utility vector */
   util_vec -> ADTUglobals = (lslong)LpiContext;         /* global area */
   util_vec -> ADTUdestroy = (lslong)destroy_commport;   /* destructor */
   util_vec -> ADTUdup     = (lslong)duplicate_commport; /* duplicator */
   util_vec -> ADTUassign  = (lslong)LNULL;              /* reserved */
   util_vec -> ADTUdisplay = (lslong)display_commport;   /* display */
   util_vec -> ADTUcompare = (lslong)compare_commport;   /* comparator */

   /* unmap the utility vector */
   LpiUnmapMem (LPI_DATA->commport_vector, VMR_0);

   /* if we got here, everything worked ok, signal success */
   LpiSignal (LPI_SUCCESS);

} /* binding proc */

/************************************************************************\
   UNBIND FUNCTION
\************************************************************************/

LpiUnbind(commport_u)
{
   /* needed to clean up from the bind routine's memory allocations.
    * This function is only called when the user removes the application
    * from memory
    */
   IgnoreLpiContext;

   /* free the utility vector */
   LpiFreeMem ( LIB_DATA->commport_vector, sizeof(LpiADTUTIL));

   /* free the library_data structure */
   FREE_REAL( LpiContext->LibData, (lslong)sizeof(library_data));
   LpiContext->LibData = LNULL;

   return (LPI_SUCCESS );
}




/************************************************************************\
   UTILITY VECTOR FUNCTIONS (called only by the Interpreter)
\************************************************************************/

/* DESTRUCTOR */
LpiDecl(destroy_commport)
{
   LpiCommportType   commport_handle;

   IgnoreCounts;
   IgnoreOutputs;

   commport_handle = LpiGetInADTdata(0);

   return( lpi_cp_destroy(LpiContext, &commport_handle ) );
}

/* DUPLICATOR */
LpiDecl(duplicate_commport)
{
   lptr(CommportType)   commport;
   LpiCommportType      commport_handle;

   IgnoreCounts;
   IgnoreOutputs;

   /* copy the input to the output */
   LpiCopyData(PLoutput(0), PLinput(0));

   commport_handle = LpiGetInADTdata(0);
   commport = LpiMapMem(commport_handle, VMR_0);

   /* if alias bit is set & refct > 1, clear the bit & bump the refct */
   if (LpiAliased(PLinput(0))) {
      if (commport -> ref_count > 1) {
         LpiClearAlias(PLinput(0));
         ++(commport->ref_count) ;
      }
   }
   else   /* alias bit not set, increment reference count */
      ++(commport->ref_count) ;

   LpiUnmapMem( commport, VMR_0 );

   return (LPI_SUCCESS);
}

/* DISPLAY */
LpiDecl(display_commport)
{
   lmhdl(lmbcs)   theString;

   IgnoreCounts;
   IgnoreOutputs;

   switch ( lpi_cp_string ( LpiContext,
                                     (LpiCommportType)LpiGetInADTdata(0),
                                     &theString ) ) {
      case LPI_SUCCESS:
         /* return a printable string */
         LpiSetOutStringH( 0, theString);
         CFreeString( LpiContext, &theString );
         LpiSignal ( LPI_SUCCESS );

      case LPI_S_OUTOFMEMORY:
         LpiSignal ( OUTOFMEMORY_TRAP );
         break;

      case LPI_S_STACK_OVERFLOW:
         LpiSignal ( STACKOVERFLOW_TRAP );
         break;

      default:
         LpiSignal ( PRIMITIVEFAILURE_TRAP );
         break;
   } /* end switch */
}

/* COMPARE */
LpiDecl (compare_commport)
{
   lptr(CommportType)   Commport_1;
   lptr(CommportType)   Commport_2;

   IgnoreParameters;

   Commport_1 = LpiMapMem(LpiGetInADTdata(0), VMR_0);
   Commport_2 = LpiMapMem(LpiGetInADTdata(1), VMR_1);

   if (memcmp (Commport_1, Commport_2,
               sizeof(CommportType)-sizeof(LpiCommportType)) == 0) {
      LpiSetOutBoolean(0, LTRUE);
   }
   else {
      LpiSetOutBoolean(0, LFALSE);
   }

   LpiUnmapMem (LpiGetInADTdata(0), VMR_0);
   LpiUnmapMem (LpiGetInADTdata(1), VMR_1);

   LpiSignal (LPI_SUCCESS);
}



/************************************************************************\
   LPL CALLABLE FUNCTIONS (constructor, attributes, procedures, operators)
\************************************************************************/
LpiDecl(construct_commport)
{
   LpiCommportType   commport_handle;
   lmhdl(LpiADTUTIL) uvect;

   IgnoreParameters;       /* avoid compiler warnings */

   switch (lpi_cp_construct(LpiContext, &commport_handle)) {

     case LPI_SUCCESS:
       uvect = LIB_DATA->commport_vector;
       LpiSetOutADT(MODULE_COMMPORT_COMMPORT,COMMPORT_COMMPORT,
                    0, commport_handle, uvect );
       LpiSignal(LPI_SUCCESS);

     case LPI_S_OUTOFMEMORY:
       LpiSignal(COMMPORT_OUTOFMEMORY);

     default:
       LpiSignal(PRIMITIVEFAILURE_TRAP);
   }
}

LpiDecl ( baudrate_set )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lslong           therate = LpiGetInInteger(1);

       IgnoreParameters;

   /* GET INSTANCE, MAKE COPY IF IT IS *NOT* ALIASED */
   /* GET_IN_ADT(p_thePort) */
   if ((!LpiAliased(PLinput(0))) &&
     (((lptr(CommportType))LpiMapMem(p_thePort, VMR_1))->ref_count > 1)) {
       LpiUnmapMem(p_thePort, VMR_1);
       lpi_cp_copy(LpiContext, p_thePort, &p_thePort);
   }


   /* CALL THE C INTERFACE FUNCTION USING SWITCH TO HANDLE RETURNS */
   switch (lpi_cp_set_baudrate(LpiContext, p_thePort, therate)) {

      case LPI_SUCCESS:
         /* IF WE WERE SUCCESSFUL AND THINGS WORKED PROPERLY */
         /* SET_OUT_ADT(p_thePort) */
         LpiCopyData(PLoutput(0),PLinput(0));  /* copy input slot to output */

         if (p_thePort != LpiGetInADTdata(0)) {/* if instance we are operating
                                                * on is not the one
                                                * that was passed in to us...
                                                */
          LpiSetOutADTdata( 0, p_thePort );  /* put the instance we operated
                                                * on into the output slot
                                                */
         }
         else {                                /* the input == operated on */
            if (!LpiAliased(PLinput(0))) {     /* if it's not aliased */
                                               /* bump the reference count
                                                * by mapping and incrementing
                                                */
               ((lptr(CommportType))LpiMapMem(p_thePort, VMR_1))->ref_count++;
               LpiUnmapMem(p_thePort, VMR_1);  /* then unmap when done */
            }
         }
         LpiSignal (LPI_SUCCESS);              /* return with success code */
         break;

      case BAUDRATE_INVALIDBAUDRATE:

         /* IF THERE WAS AN ERROR AND WE ARE NOT RETURNING THE INSTANCE */
         /* DESTROY_INSTANCE(p_thePort) */
         if (p_thePort != LpiGetInADTdata(0))  {/* if the input slot is
                                                 * not the same as the one
                                                 * we operated on, which
                                                 * means we cloned...
                                                 */
            lpi_cp_destroy( LpiContext, &p_thePort );   /* delete the copy */
         }
         LpiSignal (BAUDRATE_INVALIDBAUDRATE);
         break;
   }  /* end of switch statement */
}


LpiDecl ( baudrate_get )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lslong           therate;       /* Output parameter 0 */

       IgnoreParameters;

   switch (lpi_cp_get_baudrate(LpiContext, p_thePort, &therate)) {

      case LPI_SUCCESS:
         /* IF WE WERE SUCCESSFUL AND THINGS WORKED PROPERLY */
         LpiSetOutInteger (0, therate);
         LpiSignal (LPI_SUCCESS);
         break;

      default:
         LpiSignal (PRIMITIVEFAILURE_TRAP);
         break;
   }
}


LpiDecl ( port_set )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          thenewport = LpiGetInEnumValue(1);

       IgnoreParameters;
   GET_IN_ADT(p_thePort);

   switch ( lpi_cp_set_port (LpiContext, p_thePort, thenewport)) {

      case LPI_SUCCESS:
         SET_OUT_ADT(p_thePort)
             LpiSignal(LPI_SUCCESS);
         break;

      case PORT_INVALIDPORT:
         DESTROY_INSTANCE(p_thePort)
         LpiSignal(PORT_INVALIDPORT);
         break;
   }
}


LpiDecl ( port_get )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          thecurrentport;        /* Output parameter 0 */

       IgnoreParameters;

   switch ( lpi_cp_get_port (LpiContext, p_thePort, &thecurrentport)) {

      case LPI_SUCCESS:
       LpiSetOutEnumValue( MODULE_COMMPORT_COMMPORT, COMMPORT_PORTTYPE,
                           0, thecurrentport );
             LpiSignal(LPI_SUCCESS);
         break;

      default:
         LpiSignal(PRIMITIVEFAILURE_TRAP);
         break;
   }
}


LpiDecl ( databits_set )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          thebits = LpiGetInEnumValue(1);

       IgnoreParameters;

   GET_IN_ADT(p_thePort)

   switch ( lpi_cp_set_databits (LpiContext, p_thePort, thebits)) {
      case LPI_SUCCESS:
         SET_OUT_ADT(p_thePort)
         LpiSignal(LPI_SUCCESS);
         break;

      case DATABITS_INVALIDDATABITS:
         DESTROY_INSTANCE(p_thePort)
         LpiSignal(DATABITS_INVALIDDATABITS);
         break;
   }
}


LpiDecl ( databits_get )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          thebits;       /* Output parameter 0 */

       IgnoreParameters;

   switch ( lpi_cp_get_databits (LpiContext, p_thePort, &thebits)) {
      case LPI_SUCCESS:
       LpiSetOutEnumValue(MODULE_COMMPORT_COMMPORT,
                                 COMMPORT_DATABITTYPE, 0, thebits );
       LpiSignal(LPI_SUCCESS);
         break;

      default:
         LpiSignal(PRIMITIVEFAILURE_TRAP);
         break;
   }
}


LpiDecl ( stopbits_set )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          thebits = LpiGetInEnumValue(1);

       IgnoreParameters;
   GET_IN_ADT(p_thePort)

   switch ( lpi_cp_set_stopbits (LpiContext, p_thePort, thebits)) {
      case LPI_SUCCESS:
         SET_OUT_ADT(p_thePort)
         LpiSignal(LPI_SUCCESS);
         break;

      case STOPBITS_INVALIDSTOPBITS:
         DESTROY_INSTANCE(p_thePort)
         LpiSignal(STOPBITS_INVALIDSTOPBITS);
         break;
   }

}


LpiDecl ( stopbits_get )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          thebits;       /* Output parameter 0 */

       IgnoreParameters;

   switch ( lpi_cp_get_databits (LpiContext, p_thePort, &thebits)) {
      case LPI_SUCCESS:
       LpiSetOutEnumValue(MODULE_COMMPORT_COMMPORT,
                                       COMMPORT_STOPBITTYPE, 0, thebits );
       LpiSignal(LPI_SUCCESS);
         break;

      default:
         LpiSignal(PRIMITIVEFAILURE_TRAP);
         break;
   }

}


LpiDecl ( parity_set )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          theparity = LpiGetInEnumValue(1);

       IgnoreParameters;

   GET_IN_ADT(p_thePort)

   switch ( lpi_cp_set_parity (LpiContext, p_thePort, theparity)) {
      case LPI_SUCCESS:
         SET_OUT_ADT(p_thePort)
         LpiSignal(LPI_SUCCESS);
         break;

      default:
         DESTROY_INSTANCE(p_thePort)
         LpiSignal(PRIMITIVEFAILURE_TRAP);
         break;
   }
}


LpiDecl ( parity_get )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
       lsshort          theparity;     /* Output parameter 0 */

       IgnoreParameters;

   switch ( lpi_cp_get_parity (LpiContext, p_thePort, &theparity)) {
      case LPI_SUCCESS:
             LpiSetOutEnumValue( MODULE_COMMPORT_COMMPORT,
                  COMMPORT_PARITYTYPE, 0, theparity );
             LpiSignal(LPI_SUCCESS);
         break;

      default:
         LpiSignal(PRIMITIVEFAILURE_TRAP);
         break;
   }
}


LpiDecl ( open )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);

       IgnoreParameters;

   switch (lpi_cp_open (LpiContext, p_thePort)) {
      case LPI_SUCCESS:
         LpiSignal(LPI_SUCCESS);
         break;

      case OPEN_PORTINUSE:
         LpiSignal (OPEN_PORTINUSE);
         break;

      default:
         LpiSignal (PRIMITIVEFAILURE_TRAP);
         break;
   }
}


LpiDecl ( close )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);

       IgnoreParameters;

   switch (lpi_cp_close (LpiContext, p_thePort)) {
      case LPI_SUCCESS:
         LpiSignal(LPI_SUCCESS);
         break;

      default:
         LpiSignal (PRIMITIVEFAILURE_TRAP);
         break;
   }
}


LpiDecl ( select )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
   lmhdl(lmbcs)      theString;  /* output parameter 0 */

       IgnoreParameters;

   switch (lpi_cp_select (LpiContext, p_thePort, &theString )) {
      case LPI_SUCCESS:
         LpiSetOutStringH (0, theString);
         CFreeString(LpiContext, theString);
         LpiSignal(LPI_SUCCESS);
         break;

      case SELECT_PORTNOTOPEN:
         LpiSignal (SELECT_PORTNOTOPEN);
         break;

      case LPI_S_OUTOFMEMORY:
         LpiSignal(SELECT_OUTOFMEMORY);
         break;

      default:
         LpiSignal (PRIMITIVEFAILURE_TRAP);
         break;
   }


       LpiSignal(LPI_SUCCESS);
}


LpiDecl ( replace )
{
       LpiCommportType p_thePort = LpiGetInADTdata(0);
   lmhdl(lmbcs)      p_theString = LpiGetInString(1);

       IgnoreParameters;

   GET_IN_ADT(p_thePort)

   switch (lpi_cp_replace (LpiContext, p_thePort, p_theString)) {

      case LPI_SUCCESS:
         SET_OUT_ADT(p_thePort)
       LpiSignal(LPI_SUCCESS);
         break;

      case REPLACE_PORTNOTOPEN:
         DESTROY_INSTANCE(p_thePort)
         LpiSignal (REPLACE_PORTNOTOPEN);
         break;

      default:
         DESTROY_INSTANCE(p_thePort)
         LpiSignal (PRIMITIVEFAILURE_TRAP);
         break;
   }
}




/************************************************************************\
   C LEVEL FUNCTIONS
\************************************************************************/

/* C level destructor function */
lexpfnc (lsshort, lpi_cp_destroy, LPI_CP_DEST_ARGS)
{
   lptr(CommportType)   theCommport;

   theCommport = LpiMapMem ( *p_theCommport, VMR_0 );

   if ( --(theCommport->ref_count) == 0 ) {
      remove_from_list(LpiContext, *p_theCommport);
      LpiFreeMem ( *p_theCommport, sizeof(CommportType));
      *p_theCommport = LNULL;

   }

   return (LPI_SUCCESS);
}

/* C level constructor function */
lexpfnc (lsshort, lpi_cp_construct, LPI_CP_CONST_ARGS)
{
   lptr(CommportType)   theCommport;

   *p_theCommport = (LpiCommportType)LpiAllocMem ( CommportType,
                                                   sizeof(CommportType),
                                                   VMR_0);
   if (LpiCheckMem(*p_theCommport))    /* make sure allocated ok */
      return (COMMPORT_OUTOFMEMORY);

   theCommport = LpiMapMem ( *p_theCommport, VMR_0 );

   theCommport->ref_count     = 1;
   theCommport->baud_rate     = 1200;
   theCommport->port_number   = 1;
   theCommport->data_bits     = 8;
   theCommport->stop_bits     = 1;
   theCommport->parity        = PARITY_NONE;
   theCommport->port_open     = LFALSE;
   theCommport->next          = LNULL;
   LpiUnmapMem (*p_theCommport, VMR_0);


   /* traverse the linked list and make this last element */
   add_to_list(LpiContext, *p_theCommport);

   return (LPI_SUCCESS);
}


/* C level string function */
lexpfnc (lsshort, lpi_cp_string, LPI_CP_STRING_ARGS)
{
   theCommport;      /* avoid compiler warning */

   *p_theString = CCopyString (LpiContext, LpiLptrToLmhdl("CommPort"));

   if (p_theString == LNULL)
      return (STRING_OUTOFMEMORY);
   else
      return (LPI_SUCCESS);
}


/* C level baudrate set function */
lexpfnc (lsshort, lpi_cp_set_baudrate, LPI_CP_SET_BAUDRATE_ARGS)
{
   lptr(CommportType)   theCommport;


   switch ((int)theRate) {
      case 110:
      case 300:
      case 600:
      case 1200:
      case 2400:
      case 4800:
      case 9600:
      case 19200:
         theCommport = LpiMapMem ( p_theCommport, VMR_0 );
         theCommport->baud_rate     = (lsshort)theRate;
         if (theCommport->port_open == LTRUE) {
            /* do something to port */;
         }
         LpiUnmapMem ( p_theCommport, VMR_0 );
         break;

      default:
         return ( BAUDRATE_INVALIDBAUDRATE );
   }

   return (LPI_SUCCESS);
}


/* C level baudrate get function */
lexpfnc (lsshort, lpi_cp_get_baudrate, LPI_CP_GET_BAUDRATE_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );

   *theRate = theCommport->baud_rate;

   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level port set function */
lexpfnc (lsshort, lpi_cp_set_port, LPI_CP_SET_PORT_ARGS)
{

   lptr(CommportType)   theCommport;


   if ( thePort < PORTTYPE_COM1 || thePort > PORTTYPE_TYPE_MAX)
      return (PORT_INVALIDPORT);

   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   theCommport->port_number     = thePort;
   if (theCommport->port_open == LTRUE) {
      /* do something to port */;
   }
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level port get function */
lexpfnc (lsshort, lpi_cp_get_port, LPI_CP_GET_PORT_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );

   *thePort = theCommport->port_number;

   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}

/* C level databits set function */
lexpfnc (lsshort, lpi_cp_set_databits, LPI_CP_SET_DATABITS_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   theCommport->data_bits = (theBits == DATABITTYPE_SEVEN) ? 7 : 8;
   if (theCommport->port_open == LTRUE) {
      /* do something to port */;
   }
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level databits get function */
lexpfnc (lsshort, lpi_cp_get_databits, LPI_CP_GET_DATABITS_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   *theBits = (theCommport->data_bits  == 7) ? DATABITTYPE_SEVEN
                                                : DATABITTYPE_EIGHT;
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level stopbits set function */
lexpfnc (lsshort, lpi_cp_set_stopbits, LPI_CP_SET_STOPBITS_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   theCommport->stop_bits = theBits;
   if (theCommport->port_open == LTRUE) {
      /* do something to port */;
   }
   LpiUnmapMem ( p_theCommport, VMR_0 );


   return (LPI_SUCCESS);
}


/* C level stopbits get function */
lexpfnc (lsshort, lpi_cp_get_stopbits, LPI_CP_GET_STOPBITS_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   *theBits = theCommport->stop_bits;
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}



/* C level parity set function */
lexpfnc (lsshort, lpi_cp_set_parity, LPI_CP_SET_PARITY_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   theCommport->parity = theParity;
   if (theCommport->port_open == LTRUE) {
      /* do something to port */;
   }
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level parity get function */
lexpfnc (lsshort, lpi_cp_get_parity, LPI_CP_GET_PARITY_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   *theParity = theCommport->parity;
   if (theCommport->port_open == LTRUE) {
      /* do something to port */;
   }
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level open function */
lexpfnc (lsshort, lpi_cp_open, LPI_CP_OPEN_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   if (theCommport->port_open != LTRUE &&
      not_open(LpiContext, theCommport->port_number)) {/* if the port isn't already
                                                opened, open it */
      theCommport->port_open = LTRUE;
      theCommport->port_buffer = ALLOC_REAL( (lslong)BUFFER_LENGTH );
      strcpy (theCommport->port_buffer, "select string for test");
      /* send all the setup info to the port */

      LpiUnmapMem ( p_theCommport, VMR_0 );
   }
   else {                                    /* can't open it again */
      LpiUnmapMem ( p_theCommport, VMR_0 );
      return (OPEN_PORTINUSE);
   }

   return (LPI_SUCCESS);
}


/* C level close function */
lexpfnc (lsshort, lpi_cp_close, LPI_CP_CLOSE_ARGS)
{
   lptr(CommportType)   theCommport;


   theCommport = LpiMapMem ( p_theCommport, VMR_0 );
   if (theCommport->port_open == LTRUE ) {
      theCommport->port_open = LFALSE;          /* close it no matter what */
      FREE_REAL( theCommport->port_buffer, (lslong)BUFFER_LENGTH );
      theCommport->port_buffer = LNULL;

      /* send info to port to close it */
   }
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level select function */
lexpfnc (lsshort, lpi_cp_select, LPI_CP_SELECT_ARGS)
{
   lptr(CommportType)   theCommport;

   theCommport = LpiMapMem ( p_theCommport, VMR_0 );

   if (theCommport->port_open == LFALSE) {
      return (SELECT_PORTNOTOPEN);
      LpiUnmapMem ( p_theCommport, VMR_0 );
   }
   else {      /* write string to the port */

      /* should really extract string from port buffer */
      *theString = CCopyString (LpiContext, LpiLptrToLmhdl(theCommport->port_buffer));
      if (theString == LNULL)
         return (SELECT_OUTOFMEMORY);
   }
   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}


/* C level replace function */
lexpfnc (lsshort, lpi_cp_replace, LPI_CP_REPLACE_ARGS)
{
   lptr(CommportType)   theCommport;

   theCommport = LpiMapMem ( p_theCommport, VMR_0 );

   if (theCommport->port_open == LFALSE) {
      return (REPLACE_PORTNOTOPEN);
      LpiUnmapMem ( p_theCommport, VMR_0 );
   }
   else {      /* write string to the port */
      theString;
   }

   LpiUnmapMem ( p_theCommport, VMR_0 );

   return (LPI_SUCCESS);
}







/**************************************************************************\
   Helper Procedures
\**************************************************************************/

/* C level cloning procedure, makes a copy of the input */
lsshort static near lpi_cp_copy ( lptr(CONTEXT) LpiContext,
                                  LpiCommportType p_oldCommport,
                                  lptr(LpiCommportType) p_newCommport)
{
   lptr(CommportType)   newCommport;
   lptr(CommportType)   oldCommport;


   *p_newCommport = (LpiCommportType)LpiAllocMem ( CommportType,
                                                   sizeof(CommportType),
                                                   VMR_0);
   if (LpiCheckMem(*p_newCommport))    /* make sure allocated ok */
      return (LPI_S_OUTOFMEMORY);

   newCommport = LpiMapMem ( *p_newCommport, VMR_0 );
   oldCommport = LpiMapMem ( p_oldCommport, VMR_1 );

   memcpy ( newCommport, oldCommport, sizeof (CommportType) );
   newCommport->next = LNULL;
   newCommport->ref_count     = 1;

   LpiUnmapMem ( *p_newCommport, VMR_0 );
   LpiUnmapMem ( p_oldCommport, VMR_1 );

   add_to_list( LpiContext, *p_newCommport );

   return (LPI_SUCCESS);

}

lbool static near not_open ( lptr(CONTEXT) LpiContext,
                                  lsshort  port_number)
{
   lptr(CommportType)   theCommport;
   LpiCommportType      p_theCommport;
   lbool                retval = LTRUE;

   p_theCommport = LIB_DATA->port_list;

   while (p_theCommport != LNULL) {
      theCommport = LpiMapMem( p_theCommport, VMR_1);
      p_theCommport = theCommport->next;
      if ( theCommport->port_number == port_number &&
                                    theCommport->port_open == LTRUE ) {
         retval = LFALSE;
         p_theCommport == LNULL;
      }
   }

   return (retval);
}

void static near remove_from_list ( lptr(CONTEXT) LpiContext,
                                    LpiCommportType   p_theCommport)
{
   lptr(CommportType)      theCommport;
   lptr(CommportType)      theFollowing;
   lptr(LpiCommportType)   p_theNext;


   p_theNext = &(LIB_DATA->port_list);

   while (*p_theNext != LNULL) {
      if ( p_theCommport == *p_theNext) {       /* we found it! */
         theFollowing = LpiMapMem(p_theCommport, VMR_2);
         *p_theNext = theFollowing->next;
         LpiUnmapMem(p_theCommport, VMR_2);
      }
      else {
         LpiUnmapMem(*p_theNext, VMR_1);
         theCommport = LpiMapMem( *p_theNext, VMR_1);
         p_theNext = &(theCommport->next);

      }
   }
   return;
}


void static near add_to_list ( lptr(CONTEXT) LpiContext,
                                    LpiCommportType   p_theCommport)
{
   lptr(CommportType)      theCommport;
   LpiCommportType         p_theNext;


   if ((p_theNext = LIB_DATA->port_list) == LNULL) {
      LIB_DATA->port_list = p_theCommport;
      return;
   }

   while (p_theNext != LNULL) {
      theCommport = LpiMapMem( p_theNext, VMR_1);
      if ( (p_theNext = theCommport->next) == LNULL)  /* we found last one */
         theCommport->next = p_theCommport;
      LpiUnmapMem (p_theNext, VMR_1);
   }
   return;
}

