--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  dispdem.pl
--
--**************************************************************************

module DispDem is
-- includes a macro command procedure to set the screen display
-- to the maximum number of lines, and a library of routines to be
-- used by other add-in modules (the example module for the Window
-- ADT uses some of these routines)
use Window
use Macro    -- for MacroRun
use Register
procedure Initialize is
   Register ("MaxLines",Macro)
      except on others do
         Print ("Module DISPDEM error : MaxLines Register Failure")
      end except
end procedure Initialize
procedure ScreenSize returns (integer Cols, Rows) is
-- Returns the dimensions, in number of characters, of the
-- display; if a signal is raised, the default of 80 by 25
-- is returned.
   -- declare Display ADT
   Display [] TheScreen
      except on others do
         return (80,25)
      end except
   Cols,Rows = TheScreen.Size
end procedure ScreenSize
procedure MaxScreenSize returns (integer Cols, Rows) is
-- 1-2-3 allows for two screen device drivers, which can
-- potentially use different numbers of rows and columns. This
-- procedure uses MacroRun to set the Display to both drivers to
-- determine the largest dimensions. The procedure then ensures
-- the screen is set to the largest display mode.
   begin  -- signal block
      -- declare Display ADT
      Display [] Disp
      -- load driver 1
      MacroRun ("/WWD1")
      -- get display size
      integer D1Cols, D1Rows = Disp.Size
      -- load driver 2
      MacroRun ("/WWD2")
      -- get display size
      integer D2Cols, D2Rows = Disp.Size
      if D2Rows > D1Rows then
        return (D2Cols,D2Rows)
      else
         -- load driver 1
         MacroRun ("/WWD1")
         return (D1Cols,D1Rows)
      end if
   end -- signal block
      except on others do
         return (0,0)
      end except
end procedure MaxScreenSize
procedure MaxLines is
-- macro command procedure to set display to maximum lines
-- calls the procedure MaxScreenSize to do all the work
   integer Column,Row = MaxScreenSize
end procedure MaxLines
procedure PartDisable (Display Disp, DisplayAreaType Area)
                       returns (Disp) is
-- disables an area of the display and accommodates any signals that
-- might be raised
   begin -- signal block
      Disp.Enabled (Area) = false
         except on DisplayDisabled do
            -- another area must be disabled, enable all areas
            Disp.Enabled = true
            -- try again
            Disp.Enabled (Area) = false
         end except
   end  -- signal block
      except on InvalidUse do
         -- must be an @function procedure
      end except
end procedure PartDisable
end module DispDem
