--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  keydem.pl
--
--**************************************************************************

module KeyDem is
-- This module shows how to use the Keyboard ADT in the
-- Input library module.  It uses a pop-up window to show how
-- keyboard input can be captured and functionally interpreted.
-- Procedure PopUp, registered as a macro command, calls
-- several helper procedures to accomplish this task.
use Input     -- for KeyBoard ADT
use Window    -- for pop-up window
use Utility   -- for procedure Beep
use Register  -- to register PopUp as a macro command
procedure Initialize is
-- register procedure PopUp as a macro command
   Register("PopUp",Macro)
      except on others (string SignalMsg) do
         -- Print statement writes error message to PRINT.LOG file
         Print("KEYDEM module signaled: ",SignalMsg)
         Print("Macro registation of PopUp procedure failed")
      end except
end procedure Initialize
procedure MoveHighlight (Window TheWindow, integer Selected)
                         returns (TheWindow) is
-- restore the currently highlighted choice to its original
-- color/intensity, then highlight the newly selected choice
   begin  -- signal block
      -- set Background attribute to original color/intensity
      TheWindow.Background (CurrentArea) = Inverse
      -- restore current choice to original color/intensity
      TheWindow = Paint(TheWindow,29,1)
      -- set Background attribute to highlight color/intensity
      TheWindow.Background (CurrentArea) = Highlight
      -- position cursor at the beginning of the selected choice
      TheWindow.CursorPosition = 9, Selected
      -- highlight the selected choice
      TheWindow = Paint(TheWindow,29,1)
                  return(TheWindow)
   end
      except on others (string SignalMsg) do
         -- Print statement writes error message to PRINT.LOG file
         Print("KeyDem module - MoveHighlight procedure " ||
               "signaled: ",SignalMsg)
      end except
end procedure MoveHighlight
procedure ProcessChoice (integer ChoiceNumber) is
-- simulate processing of the choice selected by the user, by
-- displaying a window that indicates which choice was selected
   begin  -- signal block
      string ChoiceString
      Window [80,5,Wide] ProcessWin
      -- set Foreground and Background colors/intensity
      ProcessWin.Foreground (DefaultArea) = Normal
      ProcessWin.Background (DefaultArea) = Highlight
      -- assign corresponding text to string variable ChoiceString
      case ChoiceNumber
         of 2 : ChoiceString = "Enter New Tax Data"
         of 3 : ChoiceString = "Update Current Tax Records"
         of 4 : ChoiceString = "Process Tax Data"
         of 5 : ChoiceString = "Generate Status Reports"
      end case
      ProcessWin[5,2] = "You selected " || ChoiceString ||
                        " - Press any key to continue..."
      -- position the window
      ProcessWin.Position = 1,21
      -- display the window
      ProcessWin.Enabled = true
      -- pause for any keystroke
      string Pause = Input(Keyboard)
         except on others do
            -- accept and ignore any nonalphanumeric keystroke
         end except
   end
      except on others (string SignalMsg) do
         -- Print statement writes error message to PRINT.LOG file
         Print("KeyDem module - ProcessChoice procedure " ||
               "signaled: ",SignalMsg)
      end except
end procedure ProcessChoice
procedure PopUp () is
-- displays a pop-up window and lets the user navigate the
-- window using keyboard input obtained through the Keyboard ADT
   begin  -- signal block
      -- construct a 50 column x 8 row window with a single border
      Window [50, 10, Single] PopUpWin
      -- position the window
      PopUpWin.Position = 15, 8
      -- set Foreground and Background colors/intensity
      PopUpWin.Foreground (DefaultArea) = Normal
      PopUpWin.Background (DefaultArea) = Inverse
      -- assign a window title
      PopUpWin.Title = "Keyboard ADT example pop-up window"
      -- assign text to window
      PopUpWin[10,1] = ""
      PopUpWin[10,2] = "Enter New Tax Data"
      PopUpWin[10,3] = "Update Current Tax Records"
      PopUpWin[10,4] = "Process Tax Data"
      PopUpWin[10,5] = "Generate Status Reports"
      PopUpWin[10,6] = "Quit Tax Maintenance System"
      PopUpWin[10,7] = ""
      PopUpWin[ 5,8] = "Please press ENTER to make a selection"
      -- declare a constant for the row position of the first
      -- choice in the window (located in row 2 of the window)
      Constant FirstChoice = 2
      -- declare a constant for the row position of the last choice
      -- in the window (located in row 6 of the window)
      Constant LastChoice = 6
      -- declare a variable to store position of highlighted choice,
      -- initialize to start at the row position of the first choice
      integer CurrentChoice = FirstChoice
      -- highlight the current choice in the window
      PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
      -- display the window
      PopUpWin.Enabled = true
      -- process keyboard input from user until ENTER is pressed
      loop
         string Keystroke = Input(Keyboard)
            except
               -- Input procedure signals when a nonalphanumeric
               -- key is pressed.  Use these signals to control
               -- the movement of the highlight and to terminate
               -- this procedure.
               on Enter do
                  -- ENTER key was pressed, process the choice
                  if CurrentChoice <> 6 then
                     -- Quit choice was not selected
                     ProcessChoice(CurrentChoice)
                     Continue
                  else
                     -- exit the procedure
                     exit
                  end if
               on Down do  -- down arrow was pressed
                  -- increment CurrentChoice
                  if CurrentChoice = LastChoice then
                     -- the last choice is highlighted, go to
                     -- the first choice
                     CurrentChoice = FirstChoice
                  else
                     CurrentChoice = CurrentChoice + 1
                  end if
                  -- highlight the new CurrentChoice
                  PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
                  -- get a new keystroke from the user
                  continue
               on Up do  -- up arrow was pressed
                  -- increment CurrentChoice
                  if CurrentChoice = FirstChoice then
                     -- the first choice is highlighted, go to the
                     -- last choice
                     CurrentChoice = LastChoice
                  else
                     CurrentChoice = CurrentChoice - 1
                  end if
                  -- highlight the new CurrentChoice
                  PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
                  -- get a new keystroke from the user
                  continue
               on Home do  -- the HOME key was pressed
                  -- make CurrentChoice the first choice
                  CurrentChoice = FirstChoice
                  -- highlight the new CurrentChoice
                  PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
                  -- get a new keystroke from the user
                  continue
               on EndKey do  -- END key was pressed
                  -- make CurrentChoice the last choice
                  CurrentChoice = LastChoice
                  -- highlight the new CurrentChoice
                  PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
                  -- get a new keystroke from the user
                  continue
               on Break do
                  -- absolutely nothing
                  continue
               on others do
                  -- beep when the user presses any other special key
                  Beep(100,200)
                  -- get a new keystroke from the user
                  continue
            end except -- Keyboard ADT Input exception handler
         -- the user pressed an alphanumeric key
         case keystroke
            of "e","E" : -- user pressed "e" or "E"
               CurrentChoice = 2
               PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
            of "u","U" : -- user pressed "u" or "U"
               CurrentChoice = 3
               PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
            of "p","P" : -- user pressed "p" or "P"
               CurrentChoice = 4
               PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
            of "g","G" : -- user pressed "g" or "G"
               CurrentChoice = 5
               PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
            of "q","Q" : -- user pressed "q" or "Q"
               CurrentChoice = 6
               PopUpWin = MoveHighLight(PopUpWin,CurrentChoice)
            otherwise  : -- user pressed invalid alphanumeric key
                  Beep(80,200)
         end case
      end loop  -- keystroke input loop
   end -- signal block
      except on others (string SignalMsg) do
         -- Print statement writes an error message to PRINT.LOG file
         Print("KEYDEM module - PopUp signaled: ",SignalMsg)
      end except
end procedure PopUp
end module KeyDem
