--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  rangedem.pl
--
--**************************************************************************

module RangeDem is
-- RangeDem prompts the user to select a range. It then
-- recalculates the range, converts all string cells to proper
-- case, and converts all string formulas to labels. It sets
-- large negative numbers in the range to color negative, and
-- highlights large positive numbers. The range is then assigned
-- the Range Name "TempRange".
use Range
use Cell
use AtStr      -- for Proper procedure
use Utility    -- for Beep procedure

-- prologue
-- threshhold for setting ColorNegative
integer NegLevel = -100
-- threshhold for setting Highlight
integer PosLevel = 1000
-- string used to name the range in the NameTheRange procedure
string  RangeName = "TempRange"
procedure ChooseRange (Range TheRange) returns (TheRange)
                      signals (Failed) is
-- prompts the user for a range
   begin    -- of signal block
      string array [14] HelpText
      HelpText[1]="Select a range in the worksheet by"||
                  "using one of the"
      HelpText[2]="following techniques:"
      HelpText[3]=""
      HelpText[4]="1  Paint a range by moving to the top"||
                  "leftmost cell,"
      HelpText[5]="   press the anchor key (.), and expand the range."
      HelpText[6]=""
      HelpText[7]="2  Type in the cell address of"||
                  "the diagonally opposite"
      HelpText[8]="   corners of the range, and"||
                  "separate them with one"
      HelpText[9]="   or two periods, e.g. A:A1..B:F16."
      HelpText[10]=""
      HelpText[11]="3  Enter a range name."
      HelpText[12]=""
      HelpText[13]="If you have the file RANGEDEM.WK3 loaded,"
      HelpText[14]="highlight the range A3..G20."
      TheRange=Input("Select a range (F1 for help): ",false,TheRange,
                     HelpText)
   end      -- of signal block
      except on others do
         -- fail procedure on any signal
         signal Failed
      end except
end procedure ChooseRange
procedure FormatRange (Range InRange) returns (InRange) is
-- Use range array notation to check every cell
-- in the range, and adjust contents.
   begin    -- of signal block
      -- recalc range
      Calc (InRange, Rowwise)
      -- turn off protection
      InRange.Protection = false
      -- turn off highlight
      InRange.HighLight = false
      -- get first worksheet number
      integer StartSht = InRange.FirstSheet
      -- get first column
      integer StartCol = InRange.FirstColumn
      -- get first row
      integer StartRow = InRange.FirstRow
      -- get total worksheets
      integer Sht = InRange.Sheets
      -- get total columns
      integer Col = InRange.Columns
      -- get total rows
      integer Row = InRange.Rows
      -- loop through every cell in range
      for integer i = 1 to Sht loop
         for integer j = 1 to Col loop
            for integer k = 1 to Row loop
               -- check cell data type
               case InRange.DataType (i, j, k)
                  of StringData:
                     -- convert strings to proper case values
                     InRange [i, j, k] = Proper (InRange [i, j, k])
                  of NumberData:
                     -- assign Cell ADT
                     Cell [StartSht + i - 1, StartCol + j - 1,
                           StartRow + k - 1] TempCell
                     -- display large negative values in color
                     if TempCell.Value <= NegLevel then
                        TempCell.ColorNegative = true
                     -- highlight large positive values
                     elseif TempCell.Value >= PosLevel then
                        TempCell.HighLight = true
                     end if
               end case
                  except on others do
                     -- process next cell
                     continue
                  end except
            end loop  -- Row loop
         end loop  -- Col loop
      end loop  -- Sht loop
      -- turn protection on
      InRange.Protection = true
      -- set fixed format with 2 decimal places
      InRange.Format = Fixed, 2
   end      -- of signal block
      except on others do
         -- absorb any signals and end the procedure
      end except
end procedure FormatRange
procedure NameTheRange (Range InRange) returns (InRange) is
-- names the range and adds a range note
   begin    -- of signal block
      -- name the range
      NameCreate (InRange, RangeName)
      -- must create Range ADT with range name to assign note
      Range [RangeName] Temp
      -- add a note
      Temp.Note = "Created by Range Demo Add-in"
   end      -- of signal block
      except on others do
         -- absorb any signals and end procedure
      end except
end procedure NameTheRange
procedure ProcessRange is
   begin    -- of signal block
      -- declare Range ADT
      Range [] TestRange
      -- prompt user to select the range
      TestRange = ChooseRange (TestRange)
      -- modify cell settings in range
      TestRange = FormatRange (TestRange)
      -- assign the range a name and note
      TestRange = NameTheRange (TestRange)
   end  -- of signal block
      except on others do
         -- beep and end procedure
         Beep (750, 100)
      end except
end procedure ProcessRange
procedure Main Is
   -- call procedure ProcessRange
   ProcessRange
end procedure Main
end module RangeDem
