--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  toupper.pl
--
--**************************************************************************


module ToUpper is
use public RegEvent              -- Register/UnRegister Event library
#include       "EventErr"       -- Event Handler special error code defintions

#define     LowerCaseA     97    -- LMBCS code for 'a'
#define     LowerCaseZ     122   -- LMBCS code for 'z'
#define     CaseDifference 32    -- Number to add to an upper case letter
                                 -- to make it lower case

EventHandle UChandle             -- Define eventhandle so can unregister the handler

boolean     UpperCaseOn = FALSE  -- UpperCaseOn will be set to TRUE when
                                 -- the UpperCase handler is registered
----------------------------------------------------------------------------
--
-- Procedure:     UpperCase
-- Description:   Event handler for Keyboard Translate High.  If incoming key
--                is a-z, the upper case equivalent key is returned.
--
----------------------------------------------------------------------------
procedure UpperCase  (EventTrigger         theTrigger,
                      KeyTransGroup        theEvent,
                      integer              theError,
                      integer              MBCScode,
                      integer              scanCode,
                      integer              asciiCode)
          returns    (EventReturn          theReturn,
                      integer              errorCode,
                      string               errorString,
                      integer              MBCScode)
is
       -- Initialize error return values to NoError or NULL
       errorCode = NoError
       errorString = ""

   if ((MBCScode >= LowerCaseA) & (MBCScode <= LowerCaseZ)) then  -- Is key a-z ?
      MBCScode = MBCScode - CaseDifference                        -- Convert to upper
   end if

   theReturn = EventContinue
   return
end procedure UpperCase

----------------------------------------------------------------------------
--
-- Procedure:     Main
-- Description:   Toggles UpperCase on and off.
--                If UpperCaseOn = FALSE (i.e. the UpperCase event handler is
--                currently not registered), register UpperCase to be called
--                when 1-2-3 inputs keystrokes.
--                If UpperCaseOn = TRUE (the UpperCase event handler is
--                registered), then UnRegister it.
--
----------------------------------------------------------------------------
procedure Main is
   if (UpperCaseOn = FALSE) then
      UChandle = Register (KeyTransGroupHdlr (UpperCase), KeyTransHigh, BeforeEvent)
      UpperCaseOn = TRUE
   else
      UnRegister (UChandle)
      UpperCaseOn = FALSE
   end if
end procedure Main

end module ToUpper
