--**************************************************************************
--
--   Copyright (C) 1991-1994 Lotus Development Corporation.
--   All rights reserved.
--
--   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
--       the source code version and to reproduce and distribute the object
--       code version of the Example Programs provided in the Toolkit, provided
--       that you:
--
--       a) distribute the Example Programs only in conjunction with and as a
--          part of your software application product which is designed as an
--          add-in to Lotus 1-2-3, and
--
--       b) do not represent your product to be a product of Lotus Development
--          Corporation.
--
--   2.  The Example Programs are provided as is, without warranty of any kind,
--       either express or implied, including, without limitation, the implied
--       warranties of merchantability of fitness for a particular purpose.
--
--   File Name:  wfiledem.pl
--
--**************************************************************************

module WFileDem is
use WorkFile
use Utility   -- for Beep procedure
procedure SaveAndRemove (WorkFile TheFile)
   signals (NotActive, SaveFailed, RemoveFailed) is
-- Saves the worksheet file if it has been modified, and then
-- removes it from memory.
   -- if not active signal
   if TheFile.Active = NotActive then
      signal NotActive
   end if
   if TheFile.Changed = Changed then
      Save (TheFile, BackupFile)
         except on others do
            signal SaveFailed
         end except
   end if
   Delete (TheFile)
      except on others do
      -- do nothing
      end except
end procedure SaveAndRemove
procedure ActivateFile (string FileName)
            signals (failed) is
-- If the file is not active, the worksheet file is loaded from disk.
-- If the worksheet file is not on disk, a new worksheet is created.
   begin  -- signal block
      WorkFile [Filename] WFile
      if WFile.Active = Active then
         return
      end if
      FileOpen (WFile, ReserveNeeded, Following)
         except
            on FileNotFound do
               FileNew (WFile, Following)
            on others do
               signal Failed
         end except
   end   -- signal block
      except on others do
         signal Failed
      end except
end procedure ActivateFile
procedure SetMinSheets (Workfile WFile, integer TotalSheets)
   signals (Failed) is
-- Checks to see if the file has an adequate number of worksheets.
-- If the number is less than TotalSheets, additional worksheets
-- are added.
   begin  -- signal block
      -- determine current number of worksheets
      integer Sheets = WFile.Sheets
      if Sheets < TotalSheets then
         -- add worksheets so worksheet file has TotalSheets
         Insert (WFile, Sheets, TotalSheets - Sheets, Following)
      end if
   end   -- signal block
      except on others do
         signal Failed
      end except
end procedure SetMinSheets
procedure Main is
   begin  -- signal block
      -- declare an ADT object for the current worksheet file
      Workfile [] LiveFile
      -- When worksheet file SHEETDEM.WK3 is not already in
      -- memory, open it after the current worksheet file or
      -- create it.
      ActivateFile ('SheetDem')
      -- Remove the original worksheet file from memory. If it
      -- was changed, save it with a backup copy.
      SaveAndRemove (LiveFile)
   end   -- signal block
      except on others do
         -- alert the user to an error with a beep
         Beep (750,100)
         -- signal return code of 1
         Stop (1)
      end except
   -- Load TEMPFILE.WK3.  If the file does not exist, it will
   -- be created.
   string FName = 'TempFile'
   begin -- signal block
      ActivateFile (FName)
      -- declare an ADT object for TempFile
      -- make sure the file has at least four worksheets
      Workfile [FName] TheFile
      SetMinSheets (TheFile, 4)
      -- When worksheet file RANGEDEM.WK3 is not already in
      -- memory, open it after the current worksheet file or
      -- create it.
      ActivateFile ('RangeDem')
   end  -- signal block
      except on others do
         -- alert the user to an error with a beep
         Beep (750,100)
         -- signal return code of 2
         stop (2)
      end except
end procedure Main
end module WFileDem
