/***************************************************************************\

   Copyright (C) 1989-1994   Lotus Development Corporation.
   All rights reserved.

   File Name:  widthx2.c

   1.  Lotus grants you a non-exclusive royalty-free right to use and modify
       the source code version and to reproduce and distribute the object
       code version of the Example Programs provided in the Toolkit, provided
       that you:

       a) distribute the Example Programs only in conjunction with and as a
          part of your software application product which is designed as an
          add-in to Lotus 1-2-3, and

       b) do not represent your product to be a product of Lotus Development
          Corporation.

   2.  The Example Programs are provided as is, without warranty of any kind,
       either express or implied, including, without limitation, the implied
       warranties of merchantability of fitness for a particular purpose.

\***************************************************************************/


#define LPI_MAIN_MODULE

#include "widthx2.plh"
#include "lpixport.h"

#if defined(DEBUGON)
#include "debug.h"
#endif

LpiBindProto (widthx2bind);
LpiUnbindProto (widthx2unbind);
LpiProto (widthx2);

#define MAX_CELL_WIDTH  240

LpiDecl(widthx2)
{
   LpiCellType    cell_a;
   int            retval;
   lushort        the_width;

   IgnoreParameters;

   retval = LpiClConstructCurrent ( &cell_a );
   if (retval == LPI_S_OUTOFMEMORY) LpiSignal (WIDTHX2_OUTOFMEMORY);

   retval = LpiClGetWidth ( cell_a, &the_width );  /* get current width */
   switch (retval) {
      case LPI_SUCCESS:          break;
      case LPI_S_INVALID_CELL:   LpiSignal ( WIDTHX2_INVALIDCELL );
      case LPI_S_ONLY_1_WINDOW:  LpiSignal ( WIDTHX2_UNEXPECTEDERROR );
      case LPI_S_HIDDEN_COLUMN:  LpiSignal ( WIDTHX2_HIDDENCOLUMN );
   }

   the_width *= 2;      /* multiply the current width by 2 */

setwidth:
   retval = LpiClSetWidth ( cell_a, (lulong)the_width );
   switch (retval) {
      case LPI_SUCCESS:          break;
      case LPI_S_INVALID_CELL:   LpiSignal (WIDTHX2_INVALIDCELL);
      case LPI_S_ONLY_1_WINDOW:  LpiSignal (WIDTHX2_UNEXPECTEDERROR);
      case LPI_S_INVALID_USE:    LpiSignal (WIDTHX2_INVALIDUSE);
      case LPI_S_OUTOFMEMORY:    LpiSignal (WIDTHX2_OUTOFMEMORY);
      case LPI_S_FILE_SEALED:    LpiSignal (WIDTHX2_FILESEALED);
      case LPI_S_INVALID_WIDTH:  the_width = MAX_CELL_WIDTH;
                                 goto setwidth;
   }

   retval = LpiClDestroy(&cell_a);  /* ignore return code since it is
                                       ALWAYS success */

   LpiSignal( LPI_SUCCESS );
}


LpiBind(widthx2bind)
{
   /* The following 3 lines set up a table of functions which can be called
    * from LPL.
    */
   LpiSetBegin
      LpiSet(widthx2)
   LpiSetEnd

   LpiSetCode(WIDTHX2_STAMP_0, WIDTHX2_STAMP_1, WIDTHX2_VERSION );

   /***********************************************************************
    *
    * The following code loads the function tables and context information
    * for modules that have been requested at the top of the file.
    *
    *
    ***********************************************************************/

   LPI_SETUP_LOAD;   /* start loading pointers to used modules */
   if (LPI_LOAD_CELL != LPI_SUCCESS) LpiSignal (LPI_FAILURE);

#if defined(DEBUGON)
   debug();
#endif

   LpiSignal(LPI_SUCCESS);    /* everything is ok since we got here */
}


/************************************************************************
 *
 * This is the unbind routine.  It is called only when Alt-F10 Remove or
 * Alt-F10 Clear are invoked by the user.  The code in this routine should
 * work for most circumstances.
 *
 ************************************************************************/

LpiUnbind(widthx2unbind)
{
   LPI_UNLOAD;
   LpiSignal(LPI_SUCCESS);
}

